#include <iostream>
#include <stdio.h>
#include <vector>
#include <algorithm>
#include <map>

using namespace std;

#include "bt_tracker.h"
#include "topology.h"
#include "bt_peer.h"


/* Constructor function for bt_tracker*/
bt_tracker::bt_tracker()
{
	/* Using default value of 0 for tracker id */
	tracker_id = 0;
}

/* 'announce' function for bt_tracker
*  needs: 1) Pointer to peer connecting => Add it to vector
*         2) @ Returns a randomized list of peers (vector pointer) */

vector<bt_peer*>* bt_tracker::announce(bt_peer* caller)
{
	/* first check if the connecting peer is already present in list*/
	map<bt_peer*, bool>::iterator check_peer_it;
	check_peer_it = this->map_clients.find(caller);
	if(check_peer_it != this->map_clients.end())
	{
		/* this means the client already exists => no need to add it again => just return the list*/
	}
	else
	{	
		/* Add peer if not already present */
		this->clients.push_back(caller);	
		this->map_clients[caller] = true;
	}

	/* randomize list of peers (truncate if necessary) */
	/* MAY USE THE SAME LIST AS STORED */
	vector<bt_peer*>::iterator start, end;
	start = this->clients.begin();
	end = this->clients.end();

	random_shuffle(start,end);

	if(get_tracker_type() != UNIV_TRACKER)
	{
		return (&(this->clients));
	}

	else
	{

		/* this code is for biasing the neighbor selection towards univ nodes*/

		unsigned int num_to_choose = 50;

		unsigned int num_univ_clients = 0;
		unsigned int num_cm_clients = 0;

		this->ret_clients.clear();

		vector<bt_peer*>::iterator cit;
		for(cit = this->clients.begin(); cit != this->clients.end(); cit++)
		{
			bt_peer* neigh = *cit;
			if(neigh->peer_id <= num_to_choose)
			{
				if(num_univ_clients <= MAX_SELF_INITIATE / 2)
				{
					this->ret_clients.push_back(neigh);
					num_univ_clients++;
				}
				else
				{
					/* univ nodes limit reached */
					break;
				}
			}
		}
		/* this means that I have filled in the univ clients --
			now make the rest of the client as non-univ nodes */

		/* => not capping the number of non-univ nodes to any figure now*/
	

		num_cm_clients = MAX_SELF_INITIATE - num_univ_clients;
		for(cit = this->clients.begin(); cit != this->clients.end(); cit++)
		{
			bt_peer* neigh = *cit;
			if(neigh->peer_id > num_to_choose)
			{
				this->ret_clients.push_back(neigh);
			}
		}

		return (&(this->ret_clients));
	}

}

/* called when a peer is leaving the the network
 * tracker should cleanup the state for this peer
 */

void bt_tracker::peer_leaving(bt_peer* leaving_peer)
{
	map<bt_peer*, bool>::iterator pit;
	pit = this->map_clients.find(leaving_peer);
	if(pit != this->map_clients.end())
	{
		/* peer found in records*/
		this->map_clients.erase(pit);
	}
	else
	{
		fprintf(stderr,"Warning : This peer should have been in the tracker's map\n");
	}

	vector<bt_peer*>::iterator lpit;
	for(lpit = this->clients.begin(); lpit != this->clients.end(); lpit++)
	{
		/* check for the peer*/
		if(*lpit == leaving_peer)
		{
			/* remove this peer*/
			this->clients.erase(lpit);
			break;
		}
	}
}

