#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include <vector>
#include <queue>
#include "event.h"
#include "memory_pool.h"

using namespace std;

/*********************
 * CLASS DEFINITIONS *
 *********************/

class event_compare {
public:
	bool operator()(struct event *ev1, struct event *ev2)
	{
		return ev1->fire_time > ev2->fire_time;
	}
};

/****************
 * FILE GLOBALS *
 ****************/

static struct memory_pool event_pool;		/* event pool */
static priority_queue<struct event*,		/* event queue */
	vector<struct event*>,
	event_compare> event_queue;


/************************
 * FUNCTION DEFINITIONS *
 ************************/

void init_events(unsigned int num_events)
{
	assert(num_events > 0);

	init_memory_pool(&event_pool, num_events, sizeof(struct event));
}

struct event *event_alloc()
{
	return (struct event*)pool_object_alloc(&event_pool);
}

void event_free(struct event *ev)
{
	assert(ev);

	pool_object_free(&event_pool, ev);
}

struct event *event_pop()
{
	if (event_queue.empty()) {
		return NULL;
	} else {
		struct event *ret = event_queue.top();
		event_queue.pop();
		return ret;
	}
}

void event_insert(struct event *ev)
{
	assert(ev);

	event_queue.push(ev);
}
