#ifndef _EVENT_H_
#define _EVENT_H_

enum {
	E_JOIN = 1,		/* bt client joins network */
	E_LEAVE,		/* bt client leaves network */
	E_TRANSFERBLOCKS,	/* calculate block transfer event */
	E_EVALNEIGHBORS,	/* calculate client's download rate */
	E_OPTUNCHOKE,		/* optimistically unchoke */
	E_REREQUEST,		/* request neighbors from tracker */
	E_LOGGING		/* event to log data from peers */
};

class bt_peer;

struct event {
	unsigned int fire_time;	/* time at which to fire event */
	bt_peer *peer;		/* peer's event, NULL if E_TRANSFERBLOCKS */
	unsigned char type;	/* event demultiplexer */
} __attribute__ ((packed));

/**
 * Initializes the event pool and the event queue.
 * @param num_events	the number of events to allocate in the pool
 */
void init_events(unsigned int num_events);

/**
 * Obtains an event from an event pool.  Marks it as used.
 * @return an event structure, or NULL if out of available events
 */
struct event *event_alloc();

/**
 * Frees an event structure.  Marks it as unused.
 * @param ev		the event to be freed
 */
void event_free(struct event *ev);

/**
 * Obtains the next event from the event queue.  Note: does not free the event.
 * @return the next event, or NULL if there's no event in the queue
 */
struct event *event_pop();

/**
 * Inserts an event into the event queue.
 * @param ev		the event to be inserted
 */
void event_insert(struct event *ev);

#endif /* _EVENT_H_ */
