#include <assert.h>
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include "util.h"

/********************/
/* GLOBAL VARIABLES */
/********************/


static int global_debug_flag = 0;
static int global_logging_flag = 0;
static FILE *global_logfile = NULL;


/************************/
/* FUNCTION DEFINITIONS */
/************************/


void debugging_enable()
{
	assert(!global_debug_flag);

	global_debug_flag = 1;
}

void debugging_disable()
{
	assert(global_debug_flag);

	global_debug_flag = 0;
}

void debug_msg(const char *format, ...)
{
	if (!global_debug_flag) {
		return;
	}
	
	va_list arglist;
	va_start(arglist, format);
	vfprintf(stderr, format, arglist);
	va_end(arglist);
}

int open_logfile(const char *logname)
{
	assert(logname);
	assert(!global_logfile);

	global_logfile = fopen(logname, "w");
	
	return global_logfile ? 0 : 1;
}

void close_logfile()
{
	assert(global_logfile);
	
	fclose(global_logfile);
}

void logging_enable()
{
	assert(!global_logging_flag);

	global_logging_flag = 1;
}

void logging_disable()
{
	assert(global_logging_flag);

	global_logging_flag = 0;
}

void log_msg(const char *format, ...)
{
	if (!global_logging_flag) {
		return;
	}

	assert(global_logfile);
	
	va_list arglist;
	va_start(arglist, format);
	vfprintf(global_logfile, format, arglist);
	va_end(arglist);
}
